using Demo.Data.Repository;
using Demo.domain.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Demo.Domain.UseCase
{
    public class UserUseCase : IUserUseCase
    {
        private  readonly IUserRepository _repositoryUserImpl;
        private  readonly IGroupRepository _repositoryGroupImpl;

        public UserUseCase(IUserRepository userRepository, IGroupRepository groupRepository)
        {
            _repositoryUserImpl = userRepository;
            _repositoryGroupImpl = groupRepository;
        }

        public List<Group> GetAllGroups() => _repositoryGroupImpl.GetAllGroup()
        .Select(it => new Group{Id = it.Id, Name = it.Name}).ToList();

        public List<User> GetAllUsers() => _repositoryUserImpl.GetAllUser()
        .Join(_repositoryGroupImpl.GetAllGroup(),
        user => user.GroupID,
        group => group.Id,
        (user, group) => new User{
            FIO = user.FIO,
            Guid = user.Guid, 
            Group = new Group{
                Id = group.Id, 
                Name = group.Name}
            }
        ).ToList();

        public User GetUserByGuid(Guid userGuid) {
            UserLocalEntity? userLocalEntity = _repositoryUserImpl.GetUserByGuid(userGuid);
            if (userLocalEntity == null) throw new Exception("bello");
            Group? group = GetAllGroups().FirstOrDefault(it => userLocalEntity.GroupID == it.Id);
            if (group == null) throw new Exception("bello");
            return new User{
                FIO = userLocalEntity.FIO,
                Guid = userLocalEntity.Guid,
                Group = new Group{
                    Id = group.Id,
                    Name = group.Name}
            };   
        }

        public bool RemoveUserByGuid(Guid userGuid) {
            return _repositoryUserImpl.RemoveUserByGuid(userGuid);
        }
        public User UpdateUser(User user) {
            UserLocalEntity userLocalEntity = new UserLocalEntity
            {
                FIO = user.FIO,
                Guid = user.Guid,
                GroupID = user.Group.Id
            };
            UserLocalEntity? result = _repositoryUserImpl.UpdateUser(userLocalEntity);
            if (result == null)
            {
                throw new Exception("");
            }
            Group? group = GetAllGroups().FirstOrDefault(it => result.GroupID == it.Id);
            if (group == null)
            {
                throw new Exception("");
            }
            return new User
            {
                FIO = result.FIO,
                Guid = result.Guid,
                Group = group
            };
        }   
    }
}